"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const outputarea_1 = require("@jupyterlab/outputarea");
const coreutils_1 = require("@phosphor/coreutils");
const widgets_1 = require("@phosphor/widgets");
function flatten(nested) {
    return nested.reduce((res, sub) => {
        res.splice(res.length, 0, ...sub);
        return res;
    }, []);
}
/**
 * A widget hosting a cloned output area.
 */
class ClonedOutputArea extends widgets_1.Panel {
    constructor(options) {
        super();
        this._session = options.session;
        this._index = options.index !== undefined ? options.index : -1;
        this.id = `LinkedOutputView-${coreutils_1.UUID.uuid4()}`;
        this.title.label = 'Output View';
        this.title.icon = 'jp-VoilaIcon';
        this.title.caption = `For Notebook: ${this._session.notebookPath}`;
        this.addClass('jp-LinkedOutputView');
        // Wait for the notebook to be loaded before
        // cloning the output area.
        void this._session.populated.then(() => {
            const oa = flatten(this._session.outputModels)[this._index];
            if (!oa) {
                this.dispose();
                return;
            }
            const clone = new outputarea_1.SimplifiedOutputArea({
                model: oa,
                rendermime: this._session.rendermime
            });
            this.addWidget(clone);
        });
    }
    /**
     * The index of the cell in the notebook.
     */
    get index() {
        return this._index;
    }
    /**
     * The path of the notebook for the cloned output area.
     */
    get notebookPath() {
        return this._session.notebookPath;
    }
}
exports.ClonedOutputArea = ClonedOutputArea;
//# sourceMappingURL=clones.js.map