"use strict";
// Copyright (c) Vidar Tonaas Fauske
// Distributed under the terms of the Modified BSD License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/*

This module contains code for opening a new voila view.

*/
const application_1 = require("@jupyterlab/application");
const apputils_1 = require("@jupyterlab/apputils");
const apputilsExtension = __importStar(require("@jupyterlab/apputils-extension"));
const coreutils_1 = require("@jupyterlab/coreutils");
const rendermime_1 = require("@jupyterlab/rendermime");
const base = __importStar(require("@jupyter-widgets/base"));
// We import only the version from the specific module in controls so that the
// controls code can be split and dynamically loaded in webpack.
const version_1 = require("@jupyter-widgets/controls/lib/version");
const output_1 = require("./output");
const jupyterlab_manager_1 = require("@jupyter-widgets/jupyterlab-manager");
const coreutils_2 = require("@phosphor/coreutils");
const tokens_1 = require("./tokens");
const clones_1 = require("./clones");
const registry_1 = require("./registry");
const widget_1 = require("./widget");
require("../style/index.css");
const session_1 = require("./session");
const algorithm_1 = require("@phosphor/algorithm");
const originalResolver = apputilsExtension.default.find(v => v.provides === apputils_1.IWindowResolver);
const editPattern = /^\/phoila\/(?!single).+\?(.*&)*edit(&.*)*/;
const WIDGET_VIEW_MIMETYPE = 'application/vnd.jupyter.widget-view+json';
const WIDGET_REGISTRY = new registry_1.WidgetRegistry();
WIDGET_REGISTRY.register({
    name: '@jupyter-widgets/base',
    version: base.JUPYTER_WIDGETS_VERSION,
    exports: {
        WidgetModel: base.WidgetModel,
        WidgetView: base.WidgetView,
        DOMWidgetView: base.DOMWidgetView,
        DOMWidgetModel: base.DOMWidgetModel,
        LayoutModel: base.LayoutModel,
        LayoutView: base.LayoutView,
        StyleModel: base.StyleModel,
        StyleView: base.StyleView
    }
});
WIDGET_REGISTRY.register({
    name: '@jupyter-widgets/controls',
    version: version_1.JUPYTER_CONTROLS_VERSION,
    exports: () => {
        return new Promise((resolve, reject) => {
            require.ensure(['@jupyter-widgets/controls'], (require) => {
                resolve(require('@jupyter-widgets/controls'));
            }, (err) => {
                reject(err);
            }, '@jupyter-widgets/controls');
        });
    }
});
WIDGET_REGISTRY.register({
    name: '@jupyter-widgets/output',
    version: output_1.OUTPUT_WIDGET_VERSION,
    exports: { OutputModel: output_1.OutputModel, OutputView: output_1.OutputView }
});
const voilaViewPlugin = {
    id: 'phoila:voila-view',
    requires: [rendermime_1.IRenderMimeRegistry],
    optional: [apputils_1.ICommandPalette, application_1.ILayoutRestorer, rendermime_1.ILatexTypesetter],
    provides: tokens_1.TVoilaTracker,
    activate: (app, rendermime, palette, restorer, typesetter) => {
        // Workaround for https://github.com/jupyter-widgets/ipywidgets/issues/2253
        if (typesetter) {
            typesetter.typeset(document.createElement('div'));
        }
        const { commands, shell } = app;
        const sessions = {};
        const tracker = new apputils_1.WidgetTracker({ namespace: 'phoila-views' });
        const cloneTracker = new apputils_1.WidgetTracker({ namespace: 'phoila-clones' });
        commands.addCommand('phoila:open-new', {
            label: 'Open New Voila View',
            execute: (args) => __awaiter(this, void 0, void 0, function* () {
                const path = args['path'];
                const editable = args['editable'] || true;
                let ppromise;
                if (!path || typeof path !== 'string') {
                    ppromise = apputils_1.InputDialog.getText({
                        title: 'Input notebook path to open',
                        placeholder: 'Notebook path'
                    }).then((result) => {
                        if (result.button.accept && result.value) {
                            return result.value;
                        }
                        throw new Error('No valid path');
                    });
                }
                else {
                    ppromise = Promise.resolve(path);
                }
                const nbPath = yield ppromise;
                let session;
                if (!sessions[nbPath]) {
                    session = new session_1.VoilaSession(nbPath, WIDGET_REGISTRY, rendermime);
                    sessions[nbPath] = session;
                }
                else {
                    session = sessions[nbPath];
                }
                const existing = tracker.find(w => w.content.session === session);
                if (existing) {
                    existing.activate();
                    return;
                }
                // Open voila widget
                const view = new widget_1.VoilaView(session);
                view.allowDrag = editable;
                view.id = apputils_1.DOMUtils.createDomID();
                view.title.label = nbPath;
                view.title.closable = true;
                view.title.iconClass = 'jp-VoilaIcon';
                view.cloned.connect((_, clone) => {
                    cloneTracker.add(clone);
                });
                const w = new apputils_1.MainAreaWidget({
                    content: view,
                });
                w.addClass(widget_1.VOLIA_MAINAREA_CLASS);
                tracker.add(w);
                shell.add(w);
            })
        });
        commands.addCommand('phoila:clone-output', {
            execute: args => {
                const nbPath = args['notebookPath'];
                let session = sessions[nbPath];
                if (!session) {
                    session = new session_1.VoilaSession(nbPath, WIDGET_REGISTRY, rendermime);
                    sessions[nbPath] = session;
                }
                const clone = new clones_1.ClonedOutputArea({
                    session,
                    index: args['index']
                });
                const widget = new apputils_1.MainAreaWidget({
                    content: clone,
                    reveal: session.populated,
                });
                widget.addClass(widget_1.VOLIA_MAINAREA_CLASS);
                // Add the cloned output to the output widget tracker.
                cloneTracker.add(widget);
                shell.add(widget);
            }
        });
        if (restorer) {
            void restorer.restore(tracker, {
                command: 'phoila:open-new',
                args: view => ({
                    path: view.content.session.notebookPath,
                }),
                name: (view) => view.content.session.notebookPath,
                when: app.serviceManager.ready
            });
            void restorer.restore(cloneTracker, {
                command: 'phoila:clone-output',
                args: widget => ({
                    notebookPath: widget.content.notebookPath,
                    index: widget.content.index,
                }),
                name: widget => `${widget.content.notebookPath}:${widget.content.index}`,
                when: tracker.restored // After the notebook widgets (but not contents).
            });
        }
        if (palette) {
            palette.addItem({
                command: 'phoila:open-new',
                category: 'Voila',
            });
        }
        return tracker;
    },
    autoStart: true
};
const singlePattern = /^\/phoila\/single\/([^?]+)/;
const magicKey = 'phoila-single-workspace';
class CustomResolver {
    constructor() {
        this.name = magicKey;
    }
}
/**
 * Ensure we use our custom workspace name for single mode.
 */
const resolver = {
    id: 'phoila:resolver',
    autoStart: true,
    provides: apputils_1.IWindowResolver,
    requires: [application_1.JupyterFrontEnd.IPaths, application_1.IRouter],
    activate: (_, paths, router) => __awaiter(this, void 0, void 0, function* () {
        const parsed = coreutils_1.URLExt.parse(window.location.href);
        const path = parsed.pathname.replace(paths.urls.base, '/');
        if (coreutils_1.PageConfig.getOption('notebook_path') || singlePattern.test(path)) {
            return new CustomResolver();
        }
        return originalResolver.activate(_, paths, router);
    })
};
const singleModePlugin = {
    id: 'phoila:single-mode',
    requires: [application_1.IRouter, application_1.JupyterFrontEnd.IPaths, tokens_1.TVoilaTracker],
    optional: [coreutils_1.IStateDB],
    activate: (app, router, paths) => {
        const { commands } = app;
        commands.addCommand('phoila:open-single', {
            execute: (args) => __awaiter(this, void 0, void 0, function* () {
                let path = coreutils_1.PageConfig.getOption('notebook_path');
                if (!path) {
                    const match = args.path.match(singlePattern);
                    path = decodeURI(match[1]);
                }
                try {
                    yield commands.execute('phoila:open-new', { path, editable: false });
                }
                catch (error) {
                    console.warn('Single notebook routing failed.', error);
                }
                commands.execute('application:set-mode', { mode: 'single-document' });
            })
        });
        router.register({
            command: 'phoila:open-single',
            pattern: singlePattern,
        });
        if (coreutils_1.PageConfig.getOption('notebook_path')) {
            const parsed = coreutils_1.URLExt.parse(window.location.href);
            const path = parsed.pathname.replace(paths.urls.base, '/');
            if (path !== '/') {
                console.warn('Unexpected notebook_path');
            }
            else {
                void commands.execute('phoila:open-single');
            }
        }
    },
    autoStart: true
};
const autoNewWorkspacePlugin = {
    id: 'phoila:auto-edit',
    requires: [application_1.IRouter, application_1.JupyterFrontEnd.IPaths, tokens_1.TVoilaTracker],
    optional: [apputils_1.ICommandPalette, coreutils_1.IStateDB],
    activate: (app, router, paths, voilaTracker, palette) => {
        const { commands } = app;
        commands.addCommand('phoila:new-workspace', {
            label: 'New Dashboard',
            caption: 'Generate a new, automatically named workspace',
            execute: (args) => __awaiter(this, void 0, void 0, function* () {
                router.navigate(`${coreutils_1.URLExt.join(paths.urls.workspaces, coreutils_2.UUID.uuid4())}?edit`, {
                    hard: true
                });
            })
        });
        if (palette) {
            palette.addItem({
                command: 'phoila:new-workspace',
                category: 'Voila',
            });
        }
        router.register({
            command: 'phoila:new-workspace',
            pattern: /^\/phoila(|\?.*)$/,
        });
    },
    autoStart: true
};
const editModePlugin = {
    id: 'phoila:edit-mode',
    requires: [application_1.IRouter, application_1.JupyterFrontEnd.IPaths, tokens_1.TVoilaTracker],
    optional: [apputils_1.ICommandPalette, application_1.ILabShell, coreutils_1.IStateDB],
    activate: (app, router, paths, tracker, palette, labShell) => {
        const { commands } = app;
        function toggleTabBars(visible) {
            // TODO: This is a dirty hack!
            if (visible) {
                algorithm_1.each(app.shell._dockPanel.tabBars(), (tabbar) => {
                    tabbar.show();
                });
            }
            else {
                algorithm_1.each(app.shell._dockPanel.tabBars(), (tabbar) => {
                    tabbar.hide();
                });
            }
        }
        function toggleEditQueryString(value) {
            let { pathname, search, hash } = coreutils_1.URLExt.parse(window.location.href);
            path = pathname.replace(paths.urls.base, '/');
            const params = (search || '').replace(/^\?/, '')
                .split('&');
            const idx = params.indexOf('edit');
            if ((idx !== -1) === value) {
                // Already in right state
                return;
            }
            if (value) {
                params.push('edit');
            }
            else {
                params.splice(idx, 1);
            }
            router.navigate(`${path}?${params.join('&')}${hash}`);
        }
        commands.addCommand('phoila:toggle-edit-mode', {
            label: 'Edit Mode',
            isToggled: () => editMode,
            execute: () => __awaiter(this, void 0, void 0, function* () {
                editMode = !editMode;
                app.shell.toggleClass('phoila-mod-editMode', editMode);
                toggleTabBars(editMode);
                toggleEditQueryString(editMode);
                tracker.forEach(view => {
                    view.content.allowDrag = editMode;
                });
                app.shell.fit();
            })
        });
        let parsed = coreutils_1.URLExt.parse(window.location.href);
        let path = parsed.pathname.replace(paths.urls.base, '/');
        const request = path + parsed.search + parsed.hash;
        let editMode = editPattern.test(request);
        if (editMode) {
            app.shell.addClass('phoila-mod-editMode');
        }
        tracker.restored.then(() => {
            tracker.forEach(view => {
                view.content.allowDrag = editMode;
            });
        });
        app.restored.then(() => {
            if (!editMode) {
                toggleTabBars(false);
                if (labShell) {
                    // Ensure side-bars are collapsed
                    labShell.restoreLayout({
                        leftArea: {
                            collapsed: true,
                            widgets: null,
                            currentWidget: null
                        },
                        rightArea: {
                            collapsed: true,
                            widgets: null,
                            currentWidget: null
                        },
                        mainArea: null
                    });
                }
            }
        });
        if (palette) {
            palette.addItem({
                command: 'phoila:toggle-edit-mode',
                category: 'Voila',
            });
        }
    },
    autoStart: true
};
/**
 * The widget manager provider.
 */
const phoilaWidgetManagerPlugin = {
    id: 'phoila:extended-widget-manager-plugin',
    requires: [rendermime_1.IRenderMimeRegistry],
    provides: tokens_1.TPhoilaWidgetRegistry,
    activate: (app, rendermime) => {
        // Add a placeholder widget renderer.
        rendermime.addFactory({
            safe: false,
            mimeTypes: [WIDGET_VIEW_MIMETYPE],
            createRenderer: options => new jupyterlab_manager_1.WidgetRenderer(options)
        }, 0);
        return {
            registerWidget(data) {
                WIDGET_REGISTRY.register(data);
            }
        };
    },
    autoStart: true
};
/**
 * The widget manager provider.
 */
const standardWidgetManagerPlugin = {
    id: 'phoila:widget-manager-plugin',
    requires: [tokens_1.TPhoilaWidgetRegistry],
    provides: base.IJupyterWidgetRegistry,
    activate: (app, registry) => {
        return registry;
    },
    autoStart: true
};
exports.default = [
    voilaViewPlugin,
    singleModePlugin,
    editModePlugin,
    autoNewWorkspacePlugin,
    resolver,
    phoilaWidgetManagerPlugin,
    standardWidgetManagerPlugin
];
//# sourceMappingURL=plugins.js.map