"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream_helpers_1 = require("./stream-helpers");
/**
 * A multiple times replayable generator.
 *
 * Allows the elements of an async generator to be consumed by multiple
 * subscribers, that can connect at any point (before, during or after
 * element generation).
 */
class ReplayableGenerator {
    constructor(generator) {
        this._history = [];
        this._children = [];
        this._generator = generator;
        this._mainLoop();
    }
    _mainLoop() {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Await new element
                // On new element:
                // - Add to history
                // - Yield to all child gens
                for (var _b = __asyncValues(this._generator), _c; _c = yield _b.next(), !_c.done;) {
                    const element = _c.value;
                    this._history.push(element);
                    for (const child of this._children) {
                        child.enqueue(element);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    elements() {
        const that = this;
        const stream = new ReadableStream({
            start(controller) {
                that._children.push(controller);
            }
        });
        return stream_helpers_1.streamAsyncIterator(stream);
    }
}
exports.ReplayableGenerator = ReplayableGenerator;
//# sourceMappingURL=replaygen.js.map