import { IRenderMimeRegistry } from "@jupyterlab/rendermime";
import { ReadonlyJSONValue } from "@phosphor/coreutils";
import { IObservableDisposable } from '@phosphor/disposable';
import { ISignal } from '@phosphor/signaling';
import { WidgetRegistry } from './registry';
import { WidgetManager } from './widget-manager';
import { OutputAreaModel } from "@jupyterlab/outputarea";
import { ClonedOutputArea } from './clones';
export declare class VoilaSession implements IObservableDisposable {
    /**
     *
     */
    constructor(notebookPath: string, registry: WidgetRegistry, rendermime: IRenderMimeRegistry);
    /**
     * Test whether the handler is disposed.
     */
    readonly isDisposed: boolean;
    /**
     * Dispose the resources held by the handler.
     */
    dispose(): void;
    readonly entries: AsyncIterableIterator<VoilaSession.EntryData>;
    readonly connected: Promise<WidgetManager>;
    readonly populated: Promise<void>;
    /**
     * A signal emitted when the object is disposed.
     */
    readonly disposed: ISignal<this, void>;
    readonly cloneRemoved: ISignal<this, undefined>;
    readonly hasClones: boolean;
    cloneOutput(index: number): ClonedOutputArea;
    readonly notebookPath: string;
    readonly rendermime: IRenderMimeRegistry;
    readonly outputModels: ReadonlyArray<ReadonlyArray<OutputAreaModel>>;
    protected entriesFromPath(path: string): AsyncIterableIterator<VoilaSession.EntryData>;
    protected entriesFromGenerator(generator: AsyncIterableIterator<ReadonlyJSONValue>): AsyncIterableIterator<VoilaSession.EntryData>;
    protected readonly registry: WidgetRegistry;
    protected processHeader(header: VoilaSession.HeaderData): Promise<void>;
    private _isDisposed;
    private _entryGen;
    private _outputModels;
    private _connected;
    private _populated;
    private _disposed;
    private _cloneRemoved;
    private _outputClones;
}
export declare namespace VoilaSession {
    type HeaderData = {
        kernelId: string;
    };
    type EntryData = {
        source: string;
    };
    function validateEntry(entry: ReadonlyJSONValue): entry is EntryData;
    function isHeader(entry: ReadonlyJSONValue): entry is HeaderData;
}
