"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
};
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const jupyterlab_manager_1 = require("@jupyter-widgets/jupyterlab-manager");
const coreutils_1 = require("@phosphor/coreutils");
const signaling_1 = require("@phosphor/signaling");
const replaygen_1 = require("./replaygen");
const voila_1 = require("./voila");
const widget_manager_1 = require("./widget-manager");
const outputarea_1 = require("@jupyterlab/outputarea");
const clones_1 = require("./clones");
const WIDGET_VIEW_MIMETYPE = 'application/vnd.jupyter.widget-view+json';
class VoilaSession {
    /**
     *
     */
    constructor(notebookPath, registry, rendermime) {
        this._isDisposed = false;
        this._outputModels = [];
        this._connected = new coreutils_1.PromiseDelegate();
        this._populated = new coreutils_1.PromiseDelegate();
        this._disposed = new signaling_1.Signal(this);
        this._cloneRemoved = new signaling_1.Signal(this);
        this._outputClones = new Set();
        this.notebookPath = notebookPath;
        this._entryGen = new replaygen_1.ReplayableGenerator(this.entriesFromPath(notebookPath));
        this.rendermime = rendermime.clone();
        this.registry = registry;
    }
    /**
     * Test whether the handler is disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * Dispose the resources held by the handler.
     */
    dispose() {
        if (this.isDisposed) {
            return;
        }
        this._isDisposed = true;
        this._entryGen = null;
        for (const c of this._outputClones) {
            c.dispose();
        }
        this._outputClones.clear();
        for (const child of this._outputModels) {
            for (const m of child) {
                m.dispose();
            }
        }
        this._outputModels = [];
        this._disposed.emit(undefined);
        signaling_1.Signal.clearData(this);
    }
    get entries() {
        return this._entryGen.elements();
    }
    get connected() {
        return this._connected.promise;
    }
    get populated() {
        return this._populated.promise;
    }
    /**
     * A signal emitted when the object is disposed.
     */
    get disposed() {
        return this._disposed;
    }
    get cloneRemoved() {
        return this._cloneRemoved;
    }
    get hasClones() {
        return this._outputClones.size !== 0;
    }
    cloneOutput(index) {
        const clone = new clones_1.ClonedOutputArea({
            session: this,
            index
        });
        // Dispose clone when view is disposed
        this.disposed.connect(() => {
            clone.dispose();
        });
        this._outputClones.add(clone);
        clone.disposed.connect((sender) => {
            this._outputClones.delete(sender);
            this._cloneRemoved.emit(undefined);
        });
        return clone;
    }
    get outputModels() {
        return this._outputModels;
    }
    entriesFromPath(path) {
        return __asyncGenerator(this, arguments, function* entriesFromPath_1() {
            const gen = voila_1.requestVoila(path);
            try {
                yield __await(yield* __asyncDelegator(__asyncValues(this.entriesFromGenerator(gen))));
            }
            finally {
                this._populated.resolve();
            }
        });
    }
    entriesFromGenerator(generator) {
        return __asyncGenerator(this, arguments, function* entriesFromGenerator_1() {
            var e_1, _a;
            try {
                for (var generator_1 = __asyncValues(generator), generator_1_1; generator_1_1 = yield __await(generator_1.next()), !generator_1_1.done;) {
                    const entry = generator_1_1.value;
                    if (VoilaSession.isHeader(entry)) {
                        // await?
                        this.processHeader(entry);
                    }
                    if (VoilaSession.validateEntry(entry)) {
                        const rootNode = document.createElement('div');
                        rootNode.innerHTML = entry.source;
                        const outputs = rootNode.querySelectorAll('script[type="application/x.voila-lab-output+json"]');
                        const models = [];
                        for (let i = 0; i != outputs.length; ++i) {
                            const node = outputs[i];
                            const model = new outputarea_1.OutputAreaModel();
                            const data = JSON.parse(node.innerText);
                            model.fromJSON(data.outputs);
                            model.trusted = true;
                            models.push(model);
                        }
                        this._outputModels.push(models);
                        yield yield __await(entry);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (generator_1_1 && !generator_1_1.done && (_a = generator_1.return)) yield __await(_a.call(generator_1));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    processHeader(header) {
        return __awaiter(this, void 0, void 0, function* () {
            const { kernelId } = header;
            const kernel = yield voila_1.connectKernel(kernelId);
            const wManager = new widget_manager_1.WidgetManager(kernel, this.rendermime);
            this.registry.data.forEach(data => wManager.register(data));
            // Replace the placeholder widget renderer with one bound to this widget
            // manager.
            this.rendermime.removeMimeType(WIDGET_VIEW_MIMETYPE);
            this.rendermime.addFactory({
                safe: false,
                mimeTypes: [WIDGET_VIEW_MIMETYPE],
                createRenderer: (options) => new jupyterlab_manager_1.WidgetRenderer(options, wManager)
            }, 0);
            this._connected.resolve(wManager);
        });
    }
}
exports.VoilaSession = VoilaSession;
(function (VoilaSession) {
    function validateEntry(entry) {
        return (entry !== null && entry !== undefined
            && Object.keys(entry).indexOf('source') !== -1
            && typeof entry.source === 'string');
    }
    VoilaSession.validateEntry = validateEntry;
    function isHeader(entry) {
        return (entry !== null && entry !== undefined
            && Object.keys(entry).indexOf('kernelId') !== -1
            && typeof entry.kernelId === 'string');
    }
    VoilaSession.isHeader = isHeader;
})(VoilaSession = exports.VoilaSession || (exports.VoilaSession = {}));
//# sourceMappingURL=session.js.map