"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
function streamAsyncIterator(stream) {
    return __asyncGenerator(this, arguments, function* streamAsyncIterator_1() {
        const reader = stream.getReader();
        try {
            while (true) {
                const { done, value } = yield __await(reader.read());
                if (done) {
                    return yield __await(void 0);
                }
                yield yield __await(value);
            }
        }
        finally {
            reader.releaseLock();
        }
    });
}
exports.streamAsyncIterator = streamAsyncIterator;
function textDecode(input, label, options) {
    const decoder = new TextDecoder(label, options);
    const reader = input.getReader();
    return new ReadableStream({
        start(controller) {
            return __awaiter(this, void 0, void 0, function* () {
                while (true) {
                    const { done, value } = yield reader.read();
                    // When no more data needs to be consumed, break the reading
                    if (done) {
                        break;
                    }
                    controller.enqueue(decoder.decode(value));
                }
                // Close the stream
                controller.close();
                reader.releaseLock();
            });
        }
    });
}
exports.textDecode = textDecode;
/*

The following two functions have been adapted from
https://github.com/deanhume/streams under the following license:

MIT License

Copyright (c) 2019 Dean

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

*/
/**
 * Split the stream
 * @param {*} splitOn
 */
function splitStream(input, splitOn) {
    let buffer = '';
    const reader = input.getReader();
    return new ReadableStream({
        start(controller) {
            return __awaiter(this, void 0, void 0, function* () {
                while (true) {
                    const { done, value } = yield reader.read();
                    // When no more data needs to be consumed, break the reading
                    if (done) {
                        break;
                    }
                    buffer += value;
                    const parts = buffer.split(splitOn);
                    parts.slice(0, -1).forEach(part => controller.enqueue(part));
                    buffer = parts[parts.length - 1];
                }
                // Close the stream
                controller.close();
                reader.releaseLock();
            });
        }
    });
}
exports.splitStream = splitStream;
/**
 * Parse the NDJSON results
 */
function parseJSON(input) {
    const reader = input.getReader();
    return new ReadableStream({
        start(controller) {
            return __awaiter(this, void 0, void 0, function* () {
                while (true) {
                    const { done, value } = yield reader.read();
                    // When no more data needs to be consumed, break the reading
                    if (done) {
                        break;
                    }
                    if (value.trim().length > 0) {
                        controller.enqueue(JSON.parse(value));
                    }
                }
                // Close the stream
                controller.close();
                reader.releaseLock();
            });
        }
    });
}
exports.parseJSON = parseJSON;
//# sourceMappingURL=stream-helpers.js.map