import { MainAreaWidget } from '@jupyterlab/apputils';
import { OutputArea, OutputAreaModel } from '@jupyterlab/outputarea';
import { IRenderMimeRegistry } from "@jupyterlab/rendermime";
import { WidgetRenderer } from '@jupyter-widgets/jupyterlab-manager';
import { Message } from '@phosphor/messaging';
import { Widget, Layout, PanelLayout } from "@phosphor/widgets";
import { VoilaSession } from './session';
import { ISignal } from '@phosphor/signaling';
import { ClonedOutputArea } from './clones';
export declare const VOLIA_MAINAREA_CLASS = "voila-mainAreaWidget";
declare class ReplaceLayout<T extends Widget> extends Layout {
    replaceNode(target: HTMLElement, widget: T): void;
    removeWidget(widget: T): void;
    iter(): import("@phosphor/algorithm").IIterator<T>;
    _widgets: T[];
}
export declare class VoilaOutputWidget extends Widget {
    constructor(options?: Widget.IOptions);
    /**
     * Insert OutputArea for each voila output script tag in node source.
     *
     * Takes the unknown source in `node` and replaces all script tags
     * with the type "application/x.voila-lab-output+json" with a full
     * lab OutputArea.
     *
     * @param rendermime
     */
    insertOutputAreas(rendermime: IRenderMimeRegistry, outputAreaModels: ReadonlyArray<OutputAreaModel>): void;
    renderers(): IterableIterator<WidgetRenderer>;
    layout: ReplaceLayout<OutputArea>;
}
export declare class VoilaView extends Widget {
    /**
     *
     */
    constructor(session: VoilaSession);
    populateFromSession(): Promise<void>;
    addEntry(entry: VoilaSession.EntryData): void;
    allowDrag: boolean;
    readonly connected: Promise<void>;
    readonly populated: Promise<void>;
    readonly cloned: ISignal<this, MainAreaWidget<ClonedOutputArea>>;
    /**
     * Handle the DOM events for the directory listing.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the panel's DOM node. It should
     * not be called directly by user code.
     */
    handleEvent(event: Event): void;
    /**
     * A message handler invoked on an `'after-attach'` message.
     */
    protected onAfterAttach(msg: Message): void;
    /**
     * A message handler invoked on a `'before-detach'` message.
     */
    protected onBeforeDetach(msg: Message): void;
    readonly outputAreas: OutputArea[];
    private readonly _items;
    /**
     * Handle the `'mousedown'` event for the widget.
     */
    private _evtMousedown;
    /**
     * Handle the `'mouseup'` event for the widget.
     */
    private _evtMouseup;
    /**
     * Handle the `'mousemove'` event for the widget.
     */
    private _evtMousemove;
    /**
     * Start a drag event.
     */
    private _startDrag;
    layout: PanelLayout;
    readonly session: VoilaSession;
    private _allowDrag;
    private _drag;
    private _dragData;
    private _connected;
    private _populated;
    private _cloned;
}
export {};
