"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const outputarea_1 = require("@jupyterlab/outputarea");
const jupyterlab_manager_1 = require("@jupyter-widgets/jupyterlab-manager");
const algorithm_1 = require("@phosphor/algorithm");
const coreutils_1 = require("@phosphor/coreutils");
const domutils_1 = require("@phosphor/domutils");
const dragdrop_1 = require("@phosphor/dragdrop");
const messaging_1 = require("@phosphor/messaging");
const widgets_1 = require("@phosphor/widgets");
const signaling_1 = require("@phosphor/signaling");
const VOILA_VIEW_CLASS = 'voila-viewWidget';
exports.VOLIA_MAINAREA_CLASS = 'voila-mainAreaWidget';
/**
 * The factory MIME type supported by phosphor dock panels.
 */
const FACTORY_MIME = 'application/vnd.phosphor.widget-factory';
/**
 * The threshold in pixels to start a drag event.
 */
const DRAG_THRESHOLD = 5;
class ReplaceLayout extends widgets_1.Layout {
    constructor() {
        super(...arguments);
        this._widgets = [];
    }
    replaceNode(target, widget) {
        this._widgets.push(widget);
        // Send a `'before-attach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.BeforeAttach);
        }
        // Insert the widget's node before the sibling.
        target.parentElement.insertBefore(widget.node, target);
        // Send an `'after-attach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.AfterAttach);
        }
    }
    removeWidget(widget) {
        this._widgets.splice(this._widgets.indexOf(widget), 1);
        // Send a `'before-detach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.BeforeDetach);
        }
        // Remove the widget's node from the parent.
        widget.node.remove();
        // Send an `'after-detach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.AfterDetach);
        }
        if (this.parent) {
            // Post a fit request for the parent widget.
            this.parent.fit();
        }
    }
    iter() {
        return algorithm_1.iter(this._widgets);
    }
}
class VoilaOutputWidget extends widgets_1.Widget {
    constructor(options) {
        super(options);
        this.layout = new ReplaceLayout();
    }
    /**
     * Insert OutputArea for each voila output script tag in node source.
     *
     * Takes the unknown source in `node` and replaces all script tags
     * with the type "application/x.voila-lab-output+json" with a full
     * lab OutputArea.
     *
     * @param rendermime
     */
    insertOutputAreas(rendermime, outputAreaModels) {
        const rootNode = this.node;
        const outputs = rootNode.querySelectorAll('script[type="application/x.voila-lab-output+json"]');
        for (let i = 0; i != outputs.length; ++i) {
            const node = outputs[i];
            try {
                const model = outputAreaModels[i];
                const view = new outputarea_1.OutputArea({
                    model,
                    rendermime: rendermime
                });
                this.layout.replaceNode(node, view);
            }
            catch (error) {
                console.error(error);
                // Each widget view tag rendering is wrapped with a try-catch statement.
                //
                // This fixes issues with widget models that are explicitely "closed"
                // but are still referred to in a previous cell output.
                // Without the try-catch statement, this error interupts the loop and
                // prevents the rendering of further cells.
                //
                // This workaround may not be necessary anymore with templates that make use
                // of progressive rendering.
            }
        }
    }
    *renderers() {
        for (const outputArea of algorithm_1.toArray(this.layout)) {
            for (const codecell of outputArea.widgets) {
                for (const output of algorithm_1.toArray(codecell.children())) {
                    if (output instanceof jupyterlab_manager_1.WidgetRenderer) {
                        yield output;
                    }
                }
            }
        }
    }
}
exports.VoilaOutputWidget = VoilaOutputWidget;
class VoilaView extends widgets_1.Widget {
    /**
     *
     */
    constructor(session) {
        super();
        this._allowDrag = false;
        this._drag = null;
        this._dragData = null;
        this._connected = new coreutils_1.PromiseDelegate();
        this._populated = new coreutils_1.PromiseDelegate();
        this._cloned = new signaling_1.Signal(this);
        this.layout = new widgets_1.PanelLayout();
        this.addClass(VOILA_VIEW_CLASS);
        this.session = session;
        this.populateFromSession();
    }
    populateFromSession() {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.session.connected.then((wManager) => {
                    algorithm_1.each(this.layout, (widget) => {
                        for (const renderer of widget.renderers()) {
                            renderer.manager = wManager;
                        }
                    });
                });
                try {
                    for (var _b = __asyncValues(this.session.entries), _c; _c = yield _b.next(), !_c.done;) {
                        const entry = _c.value;
                        this.addEntry(entry);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            catch (reason) {
                const textnode = document.createTextNode(reason);
                this.node.appendChild(textnode);
                this._populated.reject(reason);
            }
            this._populated.resolve();
        });
    }
    addEntry(entry) {
        const view = new VoilaOutputWidget();
        view.node.innerHTML = entry.source;
        this.layout.addWidget(view);
        view.insertOutputAreas(this.session.rendermime, this.session.outputModels[this.layout.widgets.length - 1]);
    }
    get allowDrag() {
        return this._allowDrag;
    }
    set allowDrag(value) {
        this._allowDrag = value;
    }
    get connected() {
        return this._connected.promise;
    }
    get populated() {
        return this._populated.promise;
    }
    get cloned() {
        return this._cloned;
    }
    /**
     * Handle the DOM events for the directory listing.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the panel's DOM node. It should
     * not be called directly by user code.
     */
    handleEvent(event) {
        switch (event.type) {
            case 'mousedown':
                this._evtMousedown(event);
                break;
            case 'mouseup':
                this._evtMouseup(event);
                break;
            case 'mousemove':
                this._evtMousemove(event);
                break;
            default:
                break;
        }
    }
    /**
     * A message handler invoked on an `'after-attach'` message.
     */
    onAfterAttach(msg) {
        super.onAfterAttach(msg);
        let node = this.node;
        node.addEventListener('mousedown', this);
    }
    /**
     * A message handler invoked on a `'before-detach'` message.
     */
    onBeforeDetach(msg) {
        super.onBeforeDetach(msg);
        let node = this.node;
        node.removeEventListener('mousedown', this);
        document.removeEventListener('mousemove', this, true);
        document.removeEventListener('mouseup', this, true);
    }
    get outputAreas() {
        const items = [];
        for (const ow of algorithm_1.toArray(this.layout)) {
            for (const oa of algorithm_1.toArray(ow.layout)) {
                items.push(oa);
            }
        }
        return items;
    }
    get _items() {
        return this.outputAreas.map(o => o.node);
    }
    /**
     * Handle the `'mousedown'` event for the widget.
     */
    _evtMousedown(event) {
        // Left mouse press for drag start.
        if (event.button === 0 && this._allowDrag) {
            const items = this._items;
            let index = hitTestNodes(items, event.clientX, event.clientY);
            if (index === -1) {
                return;
            }
            let subtracts = [].slice.call(items[index].querySelectorAll('.jp-OutputArea-output'));
            if (hitTestNodes(subtracts, event.clientX, event.clientY) !== -1) {
                return;
            }
            this._dragData = {
                pressX: event.clientX,
                pressY: event.clientY,
                index: index
            };
            document.addEventListener('mouseup', this, true);
            document.addEventListener('mousemove', this, true);
        }
    }
    /**
     * Handle the `'mouseup'` event for the widget.
     */
    _evtMouseup(event) {
        // Remove the drag listeners if necessary.
        if (event.button !== 0 || !this._drag) {
            document.removeEventListener('mousemove', this, true);
            document.removeEventListener('mouseup', this, true);
            return;
        }
        event.preventDefault();
        event.stopPropagation();
    }
    /**
     * Handle the `'mousemove'` event for the widget.
     */
    _evtMousemove(event) {
        event.preventDefault();
        event.stopPropagation();
        // Bail if we are the one dragging.
        if (this._drag || !this._dragData) {
            return;
        }
        // Check for a drag initialization.
        let data = this._dragData;
        let dx = Math.abs(event.clientX - data.pressX);
        let dy = Math.abs(event.clientY - data.pressY);
        if (dx < DRAG_THRESHOLD && dy < DRAG_THRESHOLD) {
            return;
        }
        this._startDrag(data.index, event.clientX, event.clientY);
    }
    /**
     * Start a drag event.
     */
    _startDrag(index, clientX, clientY) {
        let source = this.outputAreas[index];
        if (!source) {
            return;
        }
        // Create the drag image.
        //let dragImage = this._items[index];
        let dragImage = document.createElement('div');
        // Set up the drag event.
        this._drag = new dragdrop_1.Drag({
            dragImage,
            mimeData: new coreutils_1.MimeData(),
            supportedActions: 'copy',
            proposedAction: 'copy'
        });
        this._drag.mimeData.setData(FACTORY_MIME, () => {
            if (!source) {
                return;
            }
            const clone = this.session.cloneOutput(index);
            const wrapper = new apputils_1.MainAreaWidget({
                content: clone
            });
            wrapper.addClass(exports.VOLIA_MAINAREA_CLASS);
            this._cloned.emit(wrapper);
            return wrapper;
        });
        // Start the drag and remove the mousemove and mouseup listeners.
        document.removeEventListener('mousemove', this, true);
        document.removeEventListener('mouseup', this, true);
        void this._drag.start(clientX, clientY).then(action => {
            this._drag = null;
        });
    }
}
exports.VoilaView = VoilaView;
function hitTestNodes(nodes, x, y) {
    return algorithm_1.ArrayExt.findFirstIndex(nodes, node => domutils_1.ElementExt.hitTest(node, x, y));
}
//# sourceMappingURL=widget.js.map